/***************************************************************************
                          p_stacked_pair.cpp  -  description
                             -------------------
    begin                : Thu Apr 11 2002
    copyright            : (C) 2002 by Mirela Andronescu
    email                : andrones@cs.ubc.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>

#include "constants.h"
#include "structs.h"
#include "externs.h"
#include "common.h"
#include "pairfold.h"
#include "p_stacked_pair.h"


p_stacked_pair::p_stacked_pair (int *seq, int link, int length)
// The constructor
{
    sequence = seq;     // just refer it from where it is in memory
    seqlen = length;
    this->V = NULL;
    this->link = link;
}



p_stacked_pair::~p_stacked_pair ()
// The destructor
{
}


int p_stacked_pair::compute_energy (int i, int j)
// compute the free energy of the structure closed by this stacked pair
{
    int min, local_energy, V_energy;
    min=INF;
    if (i+1 >= j-1) return INF;

    V_energy = V->get_energy (i+1,j-1);

    if ( i != link && j != link+1)
    {
        local_energy = stack[sequence[i]]
                            [sequence[j]]
                            [sequence[i+1]]
                            [sequence[j-1]];

        min = V_energy + local_energy;
    }
    else
    {
        min = V_energy + misc.intermolecular_initiation + 
        AU_penalty (sequence[i], sequence[j]) + 
        AU_penalty (sequence[i+1], sequence[j-1]); 
    }
    return min;

}


int p_stacked_pair::get_energy (int i, int j, int *sequence, int link)
// returns the free energy of the stacked pair closed at (i,j)   
{
    if (i+1 >= j-1) return INF;
    if ( i != link && j != link+1)
    {
        return IGINF(stack [sequence[i]]
                     [sequence[j]]
                     [sequence[i+1]]
                     [sequence[j-1]]);
    }
    return misc.intermolecular_initiation +
      AU_penalty (sequence[i], sequence[j]) + 
      AU_penalty (sequence[i+1], sequence[j-1]);  
}


int p_stacked_pair::get_enthalpy (int i, int j, int *sequence, int link)
// returns the free energy of the stacked pair closed at (i,j)   
{
    if (i+1 >= j-1) return INF;
    if ( i != link && j != link+1)
    {
        return enthalpy_stack [sequence[i]]
                              [sequence[j]]
                              [sequence[i+1]]
                              [sequence[j-1]];
    }
    return enthalpy_misc.intermolecular_initiation + 
      AU_penalty_enthalpy (sequence[i], sequence[j]) + 
      AU_penalty_enthalpy (sequence[i+1], sequence[j-1]);
}
